#!/bin/sh

. "${_EXEC}"/pdiread.sh

unescape() { [ $# = 0 ] && sed -E 's;\\(.);\1;g' || printf %s "$*" |sed -E 's;\\(.);\1;g'; }
upcase=' y;abcdefghijklmnopqrstuvwxyzäöüé;ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜÉ;; '

card_item(){
  local card="$1"
  local item cnt c
  shift 1

  for item in $@; do
    cnt="$(pdi_count "$card" "$item")"

    case $item in
      FN) printf '[h2 .item .FN &shy;%s]' "$(pdi_value "$card" FN |unescape |HTML)"
        ;;
      GENDER) printf '[span .item .GENDER &shy;%s]' "$(pdi_value "$card" GENDER |l10n)"
        ;;
      NICKNAME) seq 1 $cnt |while read c; do
          printf '[span .item .NICKNAME &shy;aka. "%s"]' \
                 "$(pdi_value "$card" NICKNAME $c |unescape |HTML)"
        done
        ;;
      X-ZACK-JOINDATE|X-ZACK-LEAVEDATE) if [ $cnt -gt 0 ]; then
          printf '[span .item .%s [b %s:] %s]' \
                 "$item" "$(l10n "${item}_short")" \
                 "$(pdi_value "$card" "$item" |HTML)"
        fi
        ;;
      BDAY) if [ $cnt -gt 0 ]; then
          printf '[span .item .BDAY [b *:] %s]' \
                 "$(pdi_value "$card" BDAY |grep -xE '[0-9-]+')"
        fi
        ;;
      SOUND) if [ $cnt -gt 0 ]; then
          printf '[audio .item .SOUND controls="controls"
                    [source type="audio/ogg" src="data:audio/ogg;base64,%s"]
                  ]' \
                 "$(pdi_value "$card" SOUND |grep -xE '[a-zA-Z0-9/+=]+')"
        fi
        ;;
      PHOTO|LOGO) if [ $cnt -gt 0 ]; then 
          printf '[img .item .%s src="data:image/%s;base64,%s"]' "$item" \
                 "$(pdi_attrib "$card" "$item" |sed -E 's;^(.*;)?TYPE="?(.+)"?(;.*)?$;\2;')" \
                 "$(pdi_value "$card" "$item" |grep -xE '[a-zA-Z0-9/+=]+')"
        fi
        ;;
      EMAIL) if [ $cnt -gt 0 ]; then
          printf '[h3 %s]' "$(l10n EMAIL)"
          seq 1 $cnt |while read c; do
            printf '[a .item .EMAIL href="mailto:%s" &shy;%s]' \
                   "$(pdi_value "$card" EMAIL $c |unescape |HTML)" \
                   "$(pdi_value "$card" EMAIL $c |unescape |HTML)"
          done
        fi
        ;;
      *) if [ $cnt -gt 0 ]; then
          printf '[h3 %s]' "$(l10n "$item")"
          seq 1 $cnt |while read c; do
            printf '[span .item .%s &shy;%s]' "$item" \
                   "$(pdi_value "$card" "$item" $c |unescape |HTML)"
          done
        fi
        ;;
    esac
  done
}

print_card(){
  local cardfile="$1"
  local cachefile="${_DATA}/cache/${cardfile##*/}.cache"

  if [ "$cachefile" -nt "$cardfile" -a "$cachefile" -nt "${_EXEC}/cards" ]; then
    cat "$cachefile"
  else
    local card="$(pdi_load "$cardfile")"
    tee "$cachefile" <<-EOF
    [div .card #${cardfile##*/}
      [div .section .basic &shy;$(
        card_item "$card" FN GENDER NICKNAME BDAY X-ZACK-JOINDATE X-ZACK-LEAVEDATE SOUND PHOTO LOGO
      )]
      [div .section .phone   &shy;$(card_item "$card" TEL)]
      [div .section .message &shy;$(card_item "$card" EMAIL IMPP URL)]
      [div .section .address &shy;$(card_item "$card" ADR)]
      [div .section .note    &shy;$(card_item "$card" NOTE)]
      [div .section .attendance [h3 $(l10n course_attendance) ] [ul &shy;
        $(grep -F "	${cardfile##*/}" "$_DATA/mappings/attendance" |while read each discard; do
          printf '[li [a .item .attendance href="/courses#%s" &shy;%s]]' \
                 "$each" \
                 "$(pdi_value "$(pdi_load "$_DATA/ical/$each")" SUMMARY |unescape |HTML)"
        done)]
        $(card_item "$card" CATEGORIES)
      ]
      [div .control
        [a .item href="/cards/${cardfile##*/}?action=edit" $(l10n edit)]
        [a .item href="/cards/${cardfile##*/}?action=export" $(l10n vcf_export)]
      ]
    ]
	EOF
  fi
}

filter_cards(){
  local filter f fex='x;p;'

  filter="$(printf %s "${filter}" \
            | sed -E 's;[]\/\(\)\\\^\$\?\.\+\*\;\[\{\}];\\\\&;g;
                      '"$upcase"
           )^"

  while [ "$filter" ]; do
    f="${filter%%^*}" filter="${filter#*^}"
    case $f in
      '') break
        ;;
      ANY:*) fex="/\n.*(\;[^:]*)?:.*(${f#*:}).*\r?\n/{${fex}}"
        ;;
      NAME:*) fex="/\n(N|FN|NICKNAME)(\;[^:]*)?:.*(${f#*:}).*\r?\n/{${fex}}"
        ;;
      STREET:*|ZIP:*) fex="/\nADR(\;[^:]*)?:.*(${f#*:}).*\r?\n/{${fex}}"
        ;;
      *) fex="/\n${f%%:*}(\;[^:]*)?:.*(${f#*:}).*\r?\n/{${fex}}"
        ;;
    esac
  done

  for cardfile in "${_DATA}"/vcard/*.vcf; do
    printf '%s\n' "$cardfile"
    cat "$cardfile"
  done \
  | sed -En ':X; /\nEND:VCARD\r?$/!{ N; bX; }; h; s;\n.*$;;; x; s;^[^\n]+\n;;;
             '"$upcase""$fex"
}

order_cards() {
  local cardfile card

  while read cardfile; do
    card="$(pdi_load "$cardfile")"

    case $order in
      firstname)
        printf '%s	%s\n' "$(pdi_value "$card" FN)" "$cardfile"
        ;;
      lastname)
        printf '%s	%s\n' "$(pdi_value "$card" N || pdi_value "$card" FN)" "$cardfile"
        ;;
      bdate)
        printf '%s	%s\n' "$(pdi_value "$card" BDAY || printf 0000-00-00)" "$cardfile"
        ;;
    esac
  done \
  | sort \
  | sed -E 's;^.*\t;;g'
}

list_cards(){
  filter_cards \
  | order_cards \
  | while read cardfile; do
    print_card "$cardfile"
  done
}
